@@ -166,9 +166,16 @@ Karll on IRC
  */
 
 #include "irc.h"
+static char cvsrevision[] = "$Id: array.c 227 2013-02-16 08:11:07Z keaston $";
+CVS_REVISION(array_c)
+
 #include "array.h"
 #include "ircaux.h"
 #include "output.h"
+#undef index
+
+#define MAIN_SOURCE
+#include "modval.h"
 
 #define ARRAY_THRESHOLD	100
 
@@ -195,13 +202,7 @@ static an_array *array_array = NULL;
  * should be inserted into the array.item[].  The function_setitem() makes use
  * of this attribute.
  */
-#ifdef __STDC__
 extern 	long		find_item (an_array array, char *find)
-#else
-extern 	long		find_item (array, find)
-	an_array     	array;
-	char *		find;
-#endif
 {
         long top, bottom, key, cmp;
 
@@ -226,19 +227,12 @@ extern 	long		find_item (array, find)
  * insert_index() takes a valid index (newIndex) and inserts it into the array
  * **index, then increments the *size of the index array.
  */
-#ifdef __STDC__
 extern 	void		insert_index (long **index, long *size, long newIndex)
-#else
-extern 	void          	insert_index (index, size, newIndex)
-	long **		index;
-	long *		size;
-	long 		newIndex;
-#endif
 {
 	long cnt;
 
 	if (*size)
-		*index = (long *)new_realloc((char *)*index, (*size + 1) * sizeof(long));
+		RESIZE(*index, long, *size + 1);
 	else
 	{
 		*index = (long *)new_malloc(sizeof(long));
@@ -255,14 +249,7 @@ extern 	void          	insert_index (index, size, newIndex)
  * move_index() moves the array.index[] up or down to make room for new entries
  * or to clean up so an entry can be deleted.
  */
-#ifdef __STDC__
 extern	void		move_index (an_array *array, long oldindex, long newindex)
-#else
-extern	void         	move_index(array, oldindex, newindex)
-	an_array *	array;
-	long		oldindex;
-	long		newindex;
-#endif
 {
 	long temp;
 
@@ -291,41 +278,30 @@ extern	void         	move_index(array, oldindex, newindex)
  * value ARRAY_THRESHOLD ought to be.  I figured someone smarter than I am
  * could figure it out and tell me or tell me to abandon the entire idea.
  */
-#ifdef __STDC__
 extern	long		find_index (an_array *array, long item)
-#else
-extern	long         	find_index (array, item)
-	an_array *	array;
-	long         	item;
-#endif
 {
-	long search = 0;
+	long i = 0;
 
 	if (array->size >= ARRAY_THRESHOLD)
 	{
-		search = find_item(*array, array->item[item]);
-		while (search >= 0 && !strcmp(array->item[array->index[search]], array->item[item]))
-			search--;
-		search++;
+		i = find_item(*array, array->item[item]);
+		while (i >= 0 && !strcmp(array->item[array->index[i]], array->item[item]))
+			i--;
+		i++;
 	}
-	while(array->index[search] != item && search < array->size)
-		search++;
+	while(array->index[i] != item && i < array->size)
+		i++;
 
-	if (search == array->size)
+	if (i == array->size)
 		say("ERROR in find_index()");	
-	return search;
+	return i;
 }
 
 /*
  * get_array() searches and finds the array referenced by *name.  It returns
  * a pointer to the array, or a null pointer on failure to find it.
  */
-#ifdef __STDC__
 extern 	an_array *	get_array (char *name)
-#else
-extern	an_array *	get_array(name)
-	char *		name;
-#endif
 {
 	long index;
 
@@ -335,20 +311,16 @@ extern	an_array *	get_array(name)
                 if ((index = find_item(array_info, name)) >= 0)
                         return &array_array[array_info.index[index]];
 	}
-	return (an_array*) 0;
+	return NULL;
 }
 
+
 /*
  * delete_array() deletes the contents of an entire array.  It assumes that
  * find_item(array_info, name) will succeed and return a valid zero or positive
  * value.
  */
-#ifdef __STDC__
 extern	void		delete_array (char *name)
-#else
-extern	void		delete_array(name)
-	char *		name;
-#endif
 {
         char **ptr;
         long cnt;
@@ -376,9 +348,9 @@ extern	void		delete_array(name)
                         *ptr = *(ptr + 1);
                         *array = *(array + 1);
                 }
-                array_info.item = (char**)new_realloc((char *)array_info.item, sizeof(char *) * array_info.size);
-                array_info.index = (long *)new_realloc((char *)array_info.index, sizeof(long) * array_info.size);
-                array_array = (an_array *)new_realloc((char *)array_array, sizeof(an_array) * array_info.size);
+                RESIZE(array_info.item, char *, array_info.size);
+                RESIZE(array_info.index, long, array_info.size);
+                RESIZE(array_array, an_array, array_info.size);
         }
         else
         {
@@ -389,6 +361,39 @@ extern	void		delete_array(name)
         }
 }
 
+void delete_all_arrays(void)
+{
+        char **ptr;
+        long index;
+        an_array *array;
+#if 0
+m_s3cat(&result, space, array_info.item[array_info.index[index]]);
+#endif	
+        for (index = 0; index < array_info.size; index++)
+        {
+        	array = &array_array[index];
+		ptr = array->item;
+		while (array->size > 0)
+		{
+        	        new_free((char **)ptr);
+			ptr++;
+			array->size--;
+		}
+	        new_free((char **)&array->item);
+        	new_free((char **)&array->index);
+	}
+	for (index = 0; index < array_info.size; index++)
+	{
+		ptr = (char **)&array_info.item[index];
+		new_free(ptr);
+		ptr++;		
+	}
+	new_free((char **)&array_info.item);
+	new_free((char **)&array_info.index);
+	new_free((char **)&array_array);
+	array_info.size = 0;
+}
+
 /*
  * Now for the actual alias functions
  * ==================================
@@ -406,18 +411,11 @@ extern	void		delete_array(name)
 #define RETURN_STR(x) return m_strdup(x ? x : EMPTY);
 #define RETURN_INT(x) return m_strdup(ltoa(x));
 
-#ifdef __STDC__
-#define BUILT_IN_FUNCTION(x, y) char * x (char * y)
-#else 
-#define BUILT_IN_FUNCTION(x, y) char * x ( y ) char * y; 
-#endif
-
-
 /*
  * function_matchitem() attempts to match a pattern to the contents of an array
  * RETURNS -1 if it cannot find the array, or -2 if no matches occur
  */
-BUILT_IN_FUNCTION(function_matchitem, input)
+BUILT_IN_FUNCTION(function_matchitem)
 {
 	char	*name;
 	long	index;
@@ -445,13 +443,58 @@ BUILT_IN_FUNCTION(function_matchitem, input)
 	RETURN_INT(match);
 }
 
+BUILT_IN_FUNCTION(function_igetmatches)
+{
+	char    *result = NULL;
+	char    *name = NULL;
+	long    item;
+	an_array *array;
+
+	if ((name = next_arg(input, &input)) &&
+		(array = get_array(name)) && *input)
+	{
+		if (*input)
+		{
+			for (item = 0; item < array->size; item++)
+			{
+				if (wild_match(input, array->item[item]) > 0)
+					m_s3cat(&result, space, ltoa(find_index(array, item)));
+			}
+		}
+	}
+
+	if (!result)
+		RETURN_EMPTY;
+
+	return result;
+}
+
+/*
+ * function_listarray() attempts to list the contents of an array
+ * RETURNS "" if it cannot find the array
+ */
+BUILT_IN_FUNCTION(function_listarray)
+{
+	char	*name;
+	an_array *array;
+	long index;
+	char *result = NULL;
+	
+	if ((name = next_arg(input, &input)) && (array = get_array(name)))
+	{
+		for (index = 0; index < array->size; index++)
+			m_s3cat(&result, space, array->item[index]);
+	}
+	return result ? result : m_strdup(empty_string);
+}
+
 /*
  * function_getmatches() attempts to match a pattern to the contents of an
  * array and returns a list of item_numbers of all items that match the pattern
  * or it returns an empty string if not items matches or if the array was not
  * found.
  */
-BUILT_IN_FUNCTION(function_getmatches, input)
+BUILT_IN_FUNCTION(function_getmatches)
 {
         char    *result = NULL;
         char    *name = NULL;
@@ -480,7 +523,7 @@ BUILT_IN_FUNCTION(function_getmatches, input)
  * patterns, much like RMATCH()
  * RETURNS -1 if it cannot find the array, or -2 if no matches occur
  */
-BUILT_IN_FUNCTION(function_rmatchitem, input)
+BUILT_IN_FUNCTION(function_rmatchitem)
 {
         char    *name = NULL;
         long    index;
@@ -513,7 +556,7 @@ BUILT_IN_FUNCTION(function_rmatchitem, input)
  * given text, or it returns a null string if no matches occur or if the array
  * was not found.
  */
-BUILT_IN_FUNCTION(function_getrmatches, input)
+BUILT_IN_FUNCTION(function_getrmatches)
 {
         char    *result = NULL;
         char    *name = NULL;
@@ -541,7 +584,7 @@ BUILT_IN_FUNCTION(function_getrmatches, input)
  * function_numitems() returns the number of items in an array, or -1 if unable
  * to find the array
  */
-BUILT_IN_FUNCTION(function_numitems, input)
+BUILT_IN_FUNCTION(function_numitems)
 {
         char *name = NULL;
 	an_array *array;
@@ -557,7 +600,7 @@ BUILT_IN_FUNCTION(function_numitems, input)
  * function_getitem() returns the value of the specified item of an array, or
  * returns an empty string on failure to find the item or array
  */
-BUILT_IN_FUNCTION(function_getitem, input)
+BUILT_IN_FUNCTION(function_getitem)
 {
 	char *name = NULL;
 	char *itemstr = NULL;
@@ -589,7 +632,7 @@ BUILT_IN_FUNCTION(function_getitem, input)
  *         -2 if it was unable to find the item (item < 0 or item was greater
  *            than 1 + the prevous maximum item number
  */
-BUILT_IN_FUNCTION(function_setitem, input)
+BUILT_IN_FUNCTION(function_setitem)
 {
         char *name = NULL;
 	char *itemstr = NULL;
@@ -617,13 +660,13 @@ BUILT_IN_FUNCTION(function_setitem, input)
 						index = find_item(*array, input);
 						index = (index >= 0) ? index : (-index) - 1;
 						move_index(array, oldindex, index);
-/*CDE Why?						new_free(&array->item[item]);*/
+						new_free(&array->item[item]);
 						malloc_strcpy(&array->item[item], input);
 						result = 0;
 					}
 					else if (item == array->size)
 					{
-						array->item = (char **)new_realloc((char *)array->item, (array->size + 1) * sizeof(char*));
+						RESIZE(array->item, char *, (array->size + 1));
 						array->item[item] = NULL;
 						malloc_strcpy(&array->item[item], input);
 						index = find_item(*array, input);
@@ -637,7 +680,7 @@ BUILT_IN_FUNCTION(function_setitem, input)
 					if (item == 0)
 					{
 						if (array_info.size)
-							array_array = (an_array*)new_realloc((char *)array_array, (array_info.size + 1) * sizeof(an_array));
+							RESIZE(array_array, an_array, (array_info.size + 1));
 						else
 							array_array = (an_array*)new_malloc(sizeof(an_array));
 						array = &array_array[array_info.size];
@@ -648,7 +691,7 @@ BUILT_IN_FUNCTION(function_setitem, input)
 						array->index[0] = 0;
 						malloc_strcpy(&array->item[0], input);
 						if (array_info.size)
-							array_info.item = (char **)new_realloc((char *)array_info.item, (array_info.size + 1) * sizeof(char *));
+							RESIZE(array_info.item, char *, (array_info.size + 1));
 						else
 							array_info.item = (char **)new_malloc(sizeof(char *));
 						array_info.item[array_info.size] = NULL;
@@ -667,13 +710,14 @@ BUILT_IN_FUNCTION(function_setitem, input)
  * function_getarrays() returns a string containg the names of all currently
  * existing arrays separated by spaces
  */
-BUILT_IN_FUNCTION(function_getarrays, input)
+BUILT_IN_FUNCTION(function_getarrays)
 {
 	long index;
 	char *result = NULL;
 
 	for (index = 0; index < array_info.size; index++)
-		m_s3cat(&result, space, array_info.item[array_info.index[index]]);
+                if (!input || !*input || wild_match(input, array_info.item[array_info.index[index]]))
+                        m_s3cat(&result, space, array_info.item[array_info.index[index]]);
 
 	if (!result)
 		RETURN_EMPTY;
@@ -684,7 +728,7 @@ BUILT_IN_FUNCTION(function_getarrays, input)
 /*
  * function_numarrays() returns the number of currently existing arrays
  */
-BUILT_IN_FUNCTION(function_numarrays, input)
+BUILT_IN_FUNCTION(function_numarrays)
 {
 	RETURN_INT(array_info.size)
 }
@@ -694,7 +738,7 @@ BUILT_IN_FUNCTION(function_numarrays, input)
  * the string that exactly matches the string searched for, or it returns
  * -1 if unable to find the array, or -2 if unable to find the item.
  */
-BUILT_IN_FUNCTION(function_finditem, input)
+BUILT_IN_FUNCTION(function_finditem)
 {
         char    *name = NULL;
         an_array *array;
@@ -716,7 +760,7 @@ BUILT_IN_FUNCTION(function_finditem, input)
  * the string that exactly matches the string searched for, or it returns
  * -1 if unable to find the array, or -2 if unable to find the item.
  */
-BUILT_IN_FUNCTION(function_ifinditem, input)
+BUILT_IN_FUNCTION(function_ifinditem)
 {
         char    *name = NULL;
         an_array *array;
@@ -738,7 +782,7 @@ BUILT_IN_FUNCTION(function_ifinditem, input)
  * or returns an empty string if unable to find the array or if the index was
  * invalid.
  */
-BUILT_IN_FUNCTION(function_igetitem, input)
+BUILT_IN_FUNCTION(function_igetitem)
 {
         char *name = NULL;
         char *itemstr = NULL;
@@ -763,7 +807,7 @@ BUILT_IN_FUNCTION(function_igetitem, input)
  * specified array.  It returns a valid item number, or -1 if unable to find
  * the array, or -2 if the index was invalid.
  */
-BUILT_IN_FUNCTION(function_indextoitem, input)
+BUILT_IN_FUNCTION(function_indextoitem)
 {
         char *name = NULL;
         char *itemstr = NULL;
@@ -789,7 +833,7 @@ BUILT_IN_FUNCTION(function_indextoitem, input)
  * refers to the item.  It returns the index number, or -1 if unable to find
  * the array, or -2 if the item was invalid.
  */
-BUILT_IN_FUNCTION(function_itemtoindex, input)
+BUILT_IN_FUNCTION(function_itemtoindex)
 {
         char *name;
         char *itemstr;
@@ -816,7 +860,7 @@ BUILT_IN_FUNCTION(function_itemtoindex, input)
  * on success, -1 if unable to find the array, -2 if unable to find the item.
  * Also, if the item is the last item in the array, it deletes the array.
  */
-BUILT_IN_FUNCTION(function_delitem, input)
+BUILT_IN_FUNCTION(function_delitem)
 {
         char *name;
 	char *itemstr;
@@ -848,8 +892,8 @@ BUILT_IN_FUNCTION(function_delitem, input)
 					array->size--;
 					for(strptr=&(array->item[item]), cnt=item; cnt < array->size; cnt++, strptr++)
 						*strptr = *(strptr + 1);
-					array->item = (char**)new_realloc((char *)array->item, sizeof(char *) * array->size);
-					array->index = (long*)new_realloc((char *)array->index, sizeof(long) * array->size);
+					RESIZE(array->item, char *, array->size);
+					RESIZE(array->index, long, array->size);
 				}
 				found = 0;
                         }
@@ -862,7 +906,7 @@ BUILT_IN_FUNCTION(function_delitem, input)
  * function_delarray() deletes the entire contents of the array using the
  * delete_array() function above.  It returns 0 on success, -1 on failure.
  */
-BUILT_IN_FUNCTION(function_delarray, input)
+BUILT_IN_FUNCTION(function_delarray)
 {
         char *name;
         long found = -1;
@@ -879,7 +923,7 @@ BUILT_IN_FUNCTION(function_delarray, input)
  * search string, or returns -2 if unable to find the array, or -1 if unable
  * to find any matches.
  */
-BUILT_IN_FUNCTION(function_ifindfirst, input)
+BUILT_IN_FUNCTION(function_ifindfirst)
 {
         char    *name;
         an_array *array;
@@ -901,3 +945,68 @@ BUILT_IN_FUNCTION(function_ifindfirst, input)
         }
 	RETURN_INT(item)
 }
+
+/*
+ * Given an array name with various strings in it, we wild_match() against
+ * the elements within the array. This allows parsing using % and * 
+ * for wildcards. We return only the best match from the array, unlike 
+ * getmatch() which returns ALL the matching items.
+ */
+
+BUILT_IN_FUNCTION(function_gettmatch)
+{
+char *name;
+an_array *array;
+char *ret = NULL;
+#if 0
+<shade> gettmatch(users % user@host *) would match the userhost mask in the
+             second word of the array
+#endif
+        if ((name = next_arg(input, &input)) && (array = get_array(name)))
+        {
+		if (*input)
+          	{
+			int index, current_match;
+			int best_match = 0;
+			int match = -1;
+                        for (index = 0; index < array->size; index++)
+                        {
+                                if ((current_match = wild_match(input, array->item[index])) > best_match)
+                                {
+                                        match = index;
+                                        best_match = current_match;
+                                }
+                        }
+			if (match != -1)
+				ret = array->item[match];
+
+		}
+	}
+	RETURN_STR(ret);
+}
+
+BUILT_IN_FUNCTION(function_igetrmatches)
+{
+	char    *result = (char *) 0;
+	char    *name = (char *) 0;
+	long    item;
+	an_array *array;
+
+	if ((name = next_arg(input, &input)) &&
+		(array = get_array(name)) && *input)
+	{
+		if (*input)
+		{
+			for (item = 0; item < array->size; item++)
+			{
+				if (wild_match(array->item[item], input) > 0)
+					m_s3cat(&result, space, ltoa(find_index(array, item)));
+			}
+		}
+	}
+
+	if (!result)
+		RETURN_EMPTY;
+
+	return result;
+}
