@@ -1,5 +1,7 @@
+#include "defs.h"
 #include "config.h"
-#if defined(NEED_GLOB) && defined(INCLUDE_GLOB_FUNCTION)
+
+#if defined(NEED_GLOB)
 /*
  * Copyright (c) 1989, 1993
  *	The Regents of the University of California.  All rights reserved.
@@ -37,7 +39,7 @@
  */
 
 /* from: static char sccsid[] = "@(#)glob.c	8.3 (Berkeley) 10/13/93"; */
-/* $Id: glob.c,v 1.3 1995/04/19 15:26:51 jnelson Exp jnelson $ */
+/* $Id: glob.c 3 2008-02-25 09:49:14Z keaston $ */
 
 /*
  * glob(3) -- a superset of the one defined in POSIX 1003.2.
@@ -65,19 +67,24 @@
  */
 
 #include <sys/param.h>
+#ifdef __EMX__
+#include <sys/types.h>
+#endif
 #include <sys/stat.h>
 
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
-#include "glob.h"
+
+#include "bsdglob.h"
 #include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include "defs.h"
 #include "ircaux.h"
+#define MAIN_SOURCE
+#include "modval.h"
 
 #ifndef MAXPATHLEN
 #define MAXPATHLEN PATHLEN
@@ -105,6 +112,7 @@
 
 #define	M_QUOTE		0x8000
 #define	M_PROTECT	0x4000
+#define M_ANYCASE       0x2000
 #define	M_MASK		0xffff
 #define	M_ASCII		0x00ff
 
@@ -133,7 +141,9 @@ static void	 	g_Ctoc (const Char *, char *);
 static int	 	g_lstat (Char *, struct stat *, glob_t *);
 static DIR	*	g_opendir (Char *, glob_t *);
 static Char	*	g_strchr (Char *, int);
+#ifdef S_ISLNK
 static int	 	g_stat (Char *, struct stat *, glob_t *);
+#endif
 static int	 	glob0 (const Char *, glob_t *);
 static int	 	glob1 (Char *, glob_t *);
 static int	 	glob2 (Char *, Char *, Char *, glob_t *);
@@ -142,9 +152,9 @@ static int	 	globextend (const Char *, glob_t *);
 static const Char *	globtilde (const Char *, Char *, glob_t *);
 static int	 	globexp1 (const Char *, glob_t *);
 static int	 	globexp2 (const Char *, const Char *, glob_t *, int *);
-static int	 	gmatch (Char *, Char *, Char *);
+static int	 	match (Char *, Char *, Char *, int);
 
-int bsd_glob		(	const char *pattern,
+int BX_bsd_glob		(	const char *pattern,
 				int flags, 
 				int (*errfunc) (const char *, int),
 				glob_t *pglob				)
@@ -153,6 +163,9 @@ int bsd_glob		(	const char *pattern,
 	int c;
 	Char *bufnext, *bufend, patbuf[MAXPATHLEN+1];
 
+#if defined(__EMX__) || defined(WINNT)
+	strlwr((char *)pattern);
+#endif
 	patnext = (u_char *) pattern;
 	if (!(flags & GLOB_APPEND)) {
 		pglob->gl_pathc = 0;
@@ -376,7 +389,9 @@ static const Char *globtilde	(	const Char *pattern,
 	/* Append the rest of the pattern */
 	while ((*b++ = *p++) != EOS)
 		continue;
-
+#if defined(__EMX__) || defined(WINNT)
+	convert_unix((char *)patbuf);
+#endif
 	return patbuf;
 }
 	
@@ -468,8 +483,11 @@ static int glob0		(	const Char *pattern,
 	       !(pglob->gl_flags & GLOB_MAGCHAR))))
 		return(globextend(pattern, pglob));
 	else if (!(pglob->gl_flags & GLOB_NOSORT)) 
-		qsort(pglob->gl_pathv + pglob->gl_offs + oldpathc,
-		    pglob->gl_pathc - oldpathc, sizeof(char *), compare);
+	{
+		if (pglob->gl_pathv)
+			qsort(pglob->gl_pathv + pglob->gl_offs + oldpathc,
+			    pglob->gl_pathc - oldpathc, sizeof(char *), compare);
+	}
 	return(0);
 }
 
@@ -564,7 +582,8 @@ static int glob3		(	Char *pathbuf,
 	DIR *dirp;
 	int err;
 	char buf[MAXPATHLEN];
-
+	int nocase = 0;
+	
 	/*
 	 * The readdirfunc declaration can't be prototyped, because it is
 	 * assigned, below, to two functions which are prototyped in glob.h
@@ -596,18 +615,28 @@ static int glob3		(	Char *pathbuf,
 		readdirfunc = pglob->gl_readdir;
 	else
 		readdirfunc = readdir;
+
+
+	if (pglob->gl_flags & GLOB_INSENSITIVE)
+		nocase = 1;
+
 	while ((dp = (*readdirfunc)(dirp))) 
 	{
 		register u_char *sc;
 		register Char *dc;
 
+#if defined(__EMX__) || defined(WINNT)
+		if(dp->d_name && *(dp->d_name))
+			strlwr(dp->d_name);
+#endif
+
 		/* Initial DOT must be matched literally. */
 		if (dp->d_name[0] == DOT && *pattern != DOT)
 			continue;
 		for (sc = (u_char *) dp->d_name, dc = pathend; 
 		     (*dc++ = *sc++) != EOS;)
 			continue;
-		if (!gmatch(pathend, pattern, restpattern)) 
+		if (!match(pathend, pattern, restpattern, nocase)) 
 		{
 			*pathend = EOS;
 			continue;
@@ -680,9 +709,10 @@ static int globextend		(	const Char *path,
  * pattern matching function for filenames.  Each occurrence of the *
  * pattern causes a recursion level.
  */
-static int gmatch		(	register Char *name,
+static int match		(	register Char *name,
 					register Char *pat,
-					register Char *patend		)
+					register Char *patend,
+					int nocase	)
 {
 	int ok, negate_range;
 	Char c, k;
@@ -696,7 +726,7 @@ static int gmatch		(	register Char *name,
 			if (pat == patend)
 				return(1);
 			do 
-			    if (gmatch(name, pat, patend))
+			    if (match(name, pat, patend, nocase))
 				    return(1);
 			while (*name++ != EOS);
 			return(0);
@@ -724,8 +754,16 @@ static int gmatch		(	register Char *name,
 				return(0);
 			break;
 		default:
-			if (*name++ != c)
-				return(0);
+			if (nocase)
+			{
+				if (toupper(CHAR(*name++)) != toupper(CHAR(c)))
+					return 0;
+			}
+			else
+			{
+				if (*name++ != c)
+					return 0;
+			}
 			break;
 		}
 	}
@@ -733,7 +771,7 @@ static int gmatch		(	register Char *name,
 }
 
 /* Free allocated data belonging to a glob_t structure. */
-void bsd_globfree 		(	glob_t *pglob			)
+void BX_bsd_globfree 		(	glob_t *pglob			)
 {
 	register int i;
 	register char **pp;
@@ -772,9 +810,14 @@ static int g_lstat		(	register Char *fn,
 	g_Ctoc(fn, buf);
 	if (pglob->gl_flags & GLOB_ALTDIRFUNC)
 		return((*pglob->gl_lstat)(buf, sb));
+#if defined(__EMX__) || defined(__OPENNT)
+	return(stat(buf, sb));
+#else
 	return(lstat(buf, sb));
+#endif
 }
 
+#ifdef S_ISLNK
 static int g_stat		(	register Char *fn,
 					struct stat *sb,
 					glob_t *pglob			)
@@ -786,6 +829,7 @@ static int g_stat		(	register Char *fn,
 		return((*pglob->gl_stat)(buf, sb));
 	return(stat(buf, sb));
 }
+#endif
 
 static Char *g_strchr		(	Char *str, 
 					int ch				)
