@@ -19,62 +19,42 @@
  *
  * And when I started getting /window create working, I discovered new
  * bugs, and there has been a few more major changes in here again.
- * It is illegal to call help from more than one screen, at the moment,
+ * It is invalid to call help from more than one screen, at the moment,
  * because there is to much to keep track of - phone, jan 1993.
  */
 
+#if 0
+static char rcsid[] = "@(#)$Id: help.c 432 2013-11-07 03:00:24Z tcava $";
+#endif
+
 #include "irc.h"
+static char cvsrevision[] = "$Id: help.c 432 2013-11-07 03:00:24Z tcava $";
+CVS_REVISION(help_c)
 
-/* stuff from gnu autoconf docs */
-
-#ifdef NeXT		/* ugly hack 'cause configure don't grok it -phone */
-# define SYSDIR
-#endif
-
-#if defined(HAVE_DIRENT_H) || defined(_POSIX_SOURCE)
-# include <dirent.h>
-#else
-# define dirent direct
-# ifdef HAVE_SYS_NDIR_H
-#  include <sys/ndir.h>
-# endif
-# ifdef HAVE_SYS_DIR_H
-#  include <sys/dir.h>
-# endif
-# ifdef HAVE_NDIR_H
-#  include <ndir.h>
-# endif
-#endif
-
-#include <sys/stat.h>
-
-#ifdef HAVE_STDARG_H
-#include <stdarg.h>
-#endif
-
+#ifdef WANT_EPICHELP
+#include "struct.h"
 #include "help.h"
 #include "input.h"
 #include "ircaux.h"
+#include "hook.h"
 #include "output.h"
 #include "screen.h"
 #include "server.h"
 #include "ircterm.h"
 #include "vars.h"
 #include "window.h"
-
-#if defined(ISC22)
-extern char *strrchr();
-# define rindex strrchr
-# include <sys/dirent.h>
-# define direct dirent
-#endif /* ISC22 */
+#include <sys/stat.h>
+#include "bsdglob.h"
+#define MAIN_SOURCE
+#include "modval.h"
 
 /* Forward declarations */
 
-static	void	help_me _((char *, char *));
-static	void	help_show_paused_topic _((char *, char *));
-static	void	create_help_window _((void));
-static	void	set_help_screen _((Screen *));
+static	void	help_me 		(char *, char *);
+static	void	help_show_paused_topic 	(char *, char *);
+static	void	create_help_window 	(void);
+static	void	set_help_screen 	(Screen *);
+static	void	help_put_it	(const char *topic, const char *format, ...);
 
 /*
  * A few variables here - A lot added to get help working with
@@ -84,54 +64,20 @@ static	void	set_help_screen _((Screen *));
 static	int	dont_pause_topic = 0;
 static	int	entry_size;
 static	int	finished_help_paging = 0;
-static	FILE *	help_fp = NULL;
-static	int	help_paused_lines;
-static	char *	help_paused_topic[256];		/* 100 should be enough */
-static	Screen *help_screen = NULL;
+static	FILE *	help_fp;
+#define HELP_PAUSED_LINES_MAX 500
+static	int	help_paused_lines = 0;
+static	char *	help_paused_topic[HELP_PAUSED_LINES_MAX]; /* Should be enough */
+static	Screen *help_screen = (Screen *) 0;
 static	int	help_show_directory = 0;
 static	char	help_topic_list[BIG_BUFFER_SIZE + 1];
-static	Window *help_window = NULL;
+static	Window *help_window = (Window *) 0;
 static	char	no_help[] = "NOHELP";
-static	char	paused_topic[128];
+static	char	paused_topic[BIG_BUFFER_SIZE + 1];
 static	char *	this_arg;
 static	int	use_help_window = 0;
 
 
-/* compar: used by scandir to alphabetize the help entries */
-static int	compar (struct dirent **e1, struct dirent **e2)
-{
-	return (my_stricmp((*e1)->d_name, (*e2)->d_name));
-}
-
-/*
- * selectent: used by scandir to decide which entries to include in the help
- * listing.  
- */
-static	int	selectent (struct dirent *entry)
-{
-	if (*(entry->d_name) == '.')
-		return (0);
-	if (my_strnicmp(entry->d_name, this_arg, strlen(this_arg)))
-		return (0);
-	else
-	{
-		int len = strlen(entry->d_name);
-		char *temp = entry->d_name;
-
-	/*
-	 * See how long the uncompressed part of the filename is..
- 	 * Before the scandir call, entry_size is set to 0, and this
-	 * right here will (eventually) set entry_size to the filename
-	 * with the longest length (not counting compression suffixes)
-	 * which will then be used as the column width.  Slick.
-	 */
-		if (! end_strcmp (temp, ".gz", 3))
-			len -=3;
-		entry_size = (len > entry_size) ? len : entry_size;
-		return (1);
-	}
-}
-
 /* 
  * show_help:  show's either a page of text from a help_fp, or the whole
  * thing, depending on the value of HELP_PAGER_VAR.  If it gets to the end,
@@ -140,47 +86,40 @@ static	int	selectent (struct dirent *entry)
  */ 
 static	int	show_help (Window *window, char *name)
 {
-	Window	*old_window;
+	Window	*old_target_window = target_window;
 	int	rows = 0;
 	char	line[256];
 
-	if (window)
-	{
-		old_window = curr_scr_win;
-		curr_scr_win = window;
-	}
-	else
-	{
-		old_window = NULL;
-		window = curr_scr_win;
-	}
+	target_window = window ? window : current_window;
 
 	if (get_int_var(HELP_PAGER_VAR))
 		rows = window->display_size;
 
-	while (--rows)
+	while (rows)
 	{
- 		if (fgets(line, 255, help_fp))
+ 		if (!fgets(line, 255, help_fp))
 		{
-			if (*(line + strlen(line) - 1) == '\n')
-			*(line + strlen(line) - 1) = (char) 0;
-
-			if (*line != '!' || *line !='#')
-				help_put_it(name, "%s", line);
-			else
-				rows++;
-		}
-		else
-		{
-			curr_scr_win = old_window;
-			if (help_fp) fclose(help_fp);
+			fclose(help_fp);
 			help_fp = NULL;
-			return (0);
+			target_window = old_target_window;
+			return 0;
 		}
+
+		if (*(line + strlen(line) - 1) == '\n')
+			*(line + strlen(line) - 1) = (char) 0;
+			
+		/*
+		 * This is for compatability with ircII-4.4
+		 */
+		if (*line == '!' || *line == '#')
+			continue;
+
+		help_put_it(name, "%s", line);
+		rows--;
 	}
-	curr_scr_win = old_window;
+
+	target_window = old_target_window;
 	return (1);
-	
 }
 
 /*
@@ -203,43 +142,48 @@ static	void	help_prompt (char *name, char *line)
 		return;
 	}
 
-	if (line && ((*line == 'q') || (*line == 'Q')))
+	if (line && toupper(*line) == 'Q')
 	{
 		finished_help_paging = 1;
-		if (help_fp) fclose(help_fp);
+#if 0
+		help_paused_lines = 0;		/* Thanks robo */
+#endif
+		fclose(help_fp);
 		help_fp = NULL;
-		set_help_screen(NULL);
+		set_help_screen((Screen *) 0);
 		return;
 	}
 
 	if (show_help(help_window, name))
 	{
-		if (dumb)
+		if (dumb_mode)
 			help_prompt(name, NULL);
 		else
 			add_wait_prompt("*** Hit any key for more, 'q' to quit ***",
-				help_prompt, name, WAIT_PROMPT_KEY);
+				help_prompt, name, WAIT_PROMPT_KEY, 1);
 	}
 	else
 	{
 		finished_help_paging = 1;
-		if (help_fp) fclose(help_fp);
+		if (help_fp)
+			fclose(help_fp);
 		help_fp = NULL;
+
 		if (help_show_directory)
 		{
 			if (get_int_var(HELP_PAGER_VAR))
 			{
-				if (dumb)
-					help_show_paused_topic(name, empty_string);
-				else
-					add_wait_prompt("*** Hit any key to end ***", 
-						help_show_paused_topic, paused_topic,
-						WAIT_PROMPT_KEY);
+			    if (dumb_mode)
+				help_show_paused_topic(name, empty_string);
+			    else
+				add_wait_prompt("*** Hit any key to end ***", 
+					help_show_paused_topic, paused_topic,
+					WAIT_PROMPT_KEY, 1);
 			}
 			else
 			{
-				help_show_paused_topic(paused_topic, empty_string);
-				set_help_screen(NULL);
+			    help_show_paused_topic(paused_topic, empty_string);
+			    set_help_screen((Screen *) 0);
 			}
 			help_show_directory = 0;
 			return;
@@ -253,16 +197,16 @@ static	void	help_prompt (char *name, char *line)
 			char	tmp[BIG_BUFFER_SIZE + 1];
 
 			sprintf(tmp, "%s%sHelp? ", help_topic_list,
-				*help_topic_list ? " " : empty_string);
-			if (!dumb)
+				*help_topic_list ? space : empty_string);
+			if (!dumb_mode)
 				add_wait_prompt(tmp, help_me, help_topic_list,
-					WAIT_PROMPT_LINE);
+					WAIT_PROMPT_LINE, 1);
 		}
 		else
 		{
 			if (*paused_topic)
 				help_show_paused_topic(paused_topic, empty_string);
-			set_help_screen(NULL);
+			set_help_screen((Screen *) 0);
 		}
 	}
 }
@@ -277,22 +221,25 @@ static	void	help_topic (char *path, char *name)
 {
 	char	*filename = NULL;
 
-	if (name == (char *) 0)
+	if (!name)
 		return;
 
 	/* what is the base name? */
-	malloc_sprintf(&filename, "%s/%s", path, name);
+	filename = m_sprintf("%s/%s", path, name);
+	if (filename[strlen(filename)-1] == '/')
+		chop(filename, 1);
 
 	/* let uzfopen have all the fun */
-	help_fp = uzfopen (&filename, path);
-	if (!help_fp)
+	if ((help_fp = uzfopen (&filename, path, 0)))
 	{
-		help_put_it (name, "%s No help available on %s: Use ? for list of topics", thing_ansi, name);
-		return;
+		/* Isnt this a heck of a lot better then the kludge you were using? */
+		help_put_it(name, "*** Help on %s", name);
+		help_prompt(name, NULL);
 	}
-	/* Isnt this a heck of a lot better then the kludge you were using? */
-	help_put_it(name, "%s Help on %s", thing_ansi, name);
-	help_prompt(name, NULL);
+	else
+		help_put_it (name, "*** No help available on %s: Use ? for list of topics", name);
+
+	new_free(&filename);
 	return;
 }
 
@@ -306,13 +253,14 @@ static	void	help_topic (char *path, char *name)
 static 	void 	help_pause_add_line (char *format, ...)
 {
 	char	buf[BIG_BUFFER_SIZE];
-
 	va_list args;
+
 	va_start (args, format);
-	vsprintf(buf, format, args);
+	vsnprintf(buf, BIG_BUFFER_SIZE - 1, format, args);
 	va_end (args);
-	malloc_strcpy(&help_paused_topic[help_paused_lines], buf);
-	help_paused_lines++;
+	if ((help_paused_lines + 1) >= HELP_PAUSED_LINES_MAX)
+		ircpanic("help_pause_add_line: would overflow the buffer");
+	malloc_strcpy(&help_paused_topic[help_paused_lines++], buf);
 }
 
 /*
@@ -328,7 +276,7 @@ static	void	help_show_paused_topic (char *name, char *line)
 	if (!help_paused_lines)
 		return;
 
-	if ((*line == 'q') || (*line == 'Q'))
+	if (toupper(*line) == 'Q')
 		i = help_paused_lines + 1;	/* just big enough */
 
 	rows = help_window->display_size;
@@ -343,16 +291,17 @@ static	void	help_show_paused_topic (char *name, char *line)
 			if (++i >= help_paused_lines)
 				break;
 		}
-		if (!dumb)
+		if (!dumb_mode)
 		{
 			if ((i < help_paused_lines) && get_int_var(HELP_PAGER_VAR))
-				add_wait_prompt("[MORE]", help_show_paused_topic, name, WAIT_PROMPT_KEY);
+				add_wait_prompt("[MORE]", help_show_paused_topic, name, WAIT_PROMPT_KEY, 1);
 		}
 		else
 			help_show_paused_topic(name, line);
 	}
 
-	/* this cant be an else of the previous if because 'i' can 
+	/* 
+	 * This cant be an else of the previous if because 'i' can 
 	 * change in the previous if and we need to test it again
 	 */
 	if (i >= help_paused_lines)
@@ -361,15 +310,15 @@ static	void	help_show_paused_topic (char *name, char *line)
 		{
 			char	buf[BIG_BUFFER_SIZE];
 
-			sprintf(buf, "%s%sHelp? ", name, (name && *name) ? " " : empty_string);
-			if (!dumb)
-				add_wait_prompt(buf, help_me, name, WAIT_PROMPT_LINE);
+			sprintf(buf, "%s%sHelp? ", name, (name && *name) ? space : empty_string);
+			if (!dumb_mode)
+				add_wait_prompt(buf, help_me, name, WAIT_PROMPT_LINE, 1);
 		}
 		else
-			set_help_screen(NULL);
+			set_help_screen((Screen *) 0);
 
 		dont_pause_topic = 0;
-		help_paused_lines = 0;
+		help_paused_lines = 0;	/* Probably should reset this ;-) */
 		i = 0;
 	}
 }
@@ -382,26 +331,26 @@ static	void	help_show_paused_topic (char *name, char *line)
 static	void	help_me (char *topics, char *args)
 {
 	char *	ptr;
-	struct	dirent	**namelist = NULL;
-	int	entries,
-		free_cnt = 0,
+	glob_t	g;
+	int	entries = 0,
 		cnt,
 		i,
 		cols;
 	struct	stat	stat_buf;
 	char	path[BIG_BUFFER_SIZE+1];
 	int	help_paused_first_call = 0;
-	char *	help_paused_path = NULL;
-	char *	help_paused_name = NULL;
+	char *	help_paused_path = (char *) 0;
+	char *	help_paused_name = (char *) 0;
 	char *	temp;
 	char	tmp[BIG_BUFFER_SIZE+1];
 	char	buffer[BIG_BUFFER_SIZE+1];
+	char *	pattern = NULL;
 
 	strcpy(help_topic_list, topics);
 	ptr = get_string_var(HELP_PATH_VAR);
 
 	sprintf(path, "%s/%s", ptr, topics);
-	for (ptr = path; (ptr = index(ptr, ' '));)
+	for (ptr = path; (ptr = strchr(ptr, ' '));)
 		*ptr = '/';
 
 	/*
@@ -418,29 +367,29 @@ static	void	help_me (char *topics, char *args)
 	finished_help_paging = 0;
 	if (access(path, R_OK|X_OK))
 	{
-		help_put_it(no_help, "%s Cannot access help directory!", thing_ansi);
-		set_help_screen(NULL);
+		help_put_it(no_help, "*** Cannot access help directory!");
+		set_help_screen((Screen *) 0);
 		return;
 	}
 
 	this_arg = next_arg(args, &args);
 	if (!this_arg && *help_topic_list && get_int_var(HELP_PROMPT_VAR))
 	{
-		if ((temp = rindex(help_topic_list, ' ')) != NULL)
+		if ((temp = strrchr(help_topic_list, ' ')) != NULL)
 			*temp = '\0';
 		else
 			*help_topic_list = '\0';
 
-		sprintf(tmp, "%s%sHelp? ", help_topic_list, *help_topic_list ? " " : empty_string);
+		sprintf(tmp, "%s%sHelp? ", help_topic_list, *help_topic_list ? space : empty_string);
 
-		if (!dumb)
-			add_wait_prompt(tmp, help_me, help_topic_list, WAIT_PROMPT_LINE);
+		if (!dumb_mode)
+			add_wait_prompt(tmp, help_me, help_topic_list, WAIT_PROMPT_LINE, 1);
 		return;
 	}
 
 	if (!this_arg)
 	{
-		set_help_screen(NULL);
+		set_help_screen((Screen *) 0);
 		return;
 	}
 
@@ -455,7 +404,8 @@ static	void	help_me (char *topics, char *args)
 	 */
 	while (this_arg)
 	{
-		message_from(NULL, LOG_CURRENT);
+		entries = 0;
+		reset_display_target();
 
 		if (!*this_arg)
 			help_topic(path, NULL);
@@ -468,62 +418,75 @@ static	void	help_me (char *topics, char *args)
 		}
 
 		/*
-		 * entry_size is set by our scandir() handler to the width
-		 * of the longest help topic (adjusted for compression
-		 * extensions, of course.)
+		 * entry_size is set to the width of the longest help topic
+		 * (adjusted for compression extensions, of course.)
 		 */
 		entry_size = 0;
 
 		/*
 		 * Gather up the names of the files in the help directory.
-		 * XXXX - this should be done by glob().  Scandir() isnt
-		 *        standarized.
 		 */
-		free_cnt = entries = scandir(path, &namelist, 
-			(int (*) _((const struct dirent *))) selectent,
-			(const void *)(int (*) _((const struct dirent *, const struct dirent *)))compar);
+		{
+#ifndef HAVE_FCHDIR
+			char 	opath[MAXPATHLEN + 1];
+			getcwd(opath, MAXPATHLEN);
+#else
+			int 	cwd = open(".", O_RDONLY);
+#endif
+
+			chdir(path);
+			pattern = alloca(strlen(path) + 2 + 
+					 strlen(this_arg) + 3);
+			strcpy(pattern, this_arg);
+			strcat(pattern, "*");
+#ifdef GLOB_INSENSITIVE
+			bsd_glob(pattern, GLOB_INSENSITIVE /* GLOB_MARK */, NULL, &g);
+#else
+			bsd_glob(pattern, 0 /* GLOB_MARK */, NULL, &g);
+#endif
+#ifndef HAVE_FCHDIR
+			chdir(opath);
+#else
+			fchdir(cwd);
+			close(cwd);
+#endif
+		}
+
+		for (i = 0; i < g.gl_matchc; i++)
+		{
+			char	*tmp = g.gl_pathv[i];
+			int 	len = strlen(tmp);
+
+			if (!end_strcmp(tmp, ".gz", 3))
+				len -= 3;
+			else if (!end_strcmp(tmp, ".bz2", 4))
+				len -= 4;
+			entry_size = (len > entry_size) ? len : entry_size;
+		}
 
 		/*
-		 * Because there are help files which are a full subset
-		 * of a the name of another help file (ie, 'LOG' and 
-		 * 'LOGFILE'), we need to make sure that if the user said
-		 * /help LOG, they really get the help file for LOG and are
-		 * not asked to pick between LOG and LOGFILE.
-		 *
-		 * So:  If there is more then one entry, but one of the
-		 * filenames is exactly what the user requested (adjusted
-		 * for compression extension, of course), then we use it
-		 * and forget the others.
+		 * Right here we need to check for an 'exact match'.
+		 * An 'exact match' would be sitting in gl_pathv[0],
+		 * and it is 'exact' if it is identical to what we are
+		 * looking for, or if it is the same except that it has
+		 * a compression extension on it
 		 */
-		if (entries > 1)
+		if (g.gl_matchc > 1)
 		{
-			/*
-			 * Because the filenames are sorted, we know that
-			 * the help topic that the user requested would
-			 * definitely be in namelist[0].
-			 *
-			 * First thing we do is see how many characters
-			 * are different between namelist[0] and the topic
-			 * we're looking for.
-			 */
-			int len1 = strlen (namelist[0]->d_name);
-			int len2 = strlen (this_arg);
-			int len3 = len1 - len2;	/* should be positive */
+			char *str1 = g.gl_pathv[0];
+			char *str2 = this_arg;
+			int len1 = strlen(str1);
+			int len2 = strlen(str2);
 
-			/*
-			 * If namelist[0] and the help topic match to
-			 * the length of the help topic, then if the
-			 * difference is exactly the length of a compression
-			 * suffix AND that suffix is present, then we have
-			 * the topic we're looking for.  We clean off
-			 * all the other topics and reset the size.
-			 */
-			if ((!my_strnicmp(namelist[0]->d_name, this_arg, len2)
-				&& !end_strcmp(namelist[0]->d_name, ".gz", len3))
-			   || !my_stricmp(namelist[0]->d_name, this_arg))
-			{
+
+			     if (len1 == len2 && !my_stricmp(str1, str2))
+				entries = 1;
+			else if (len1 - 3 == len2 && !my_strnicmp(str1, str2, len2) && !end_strcmp(str1, ".gz", 3))
+				entries = 1;
+			else if (len1 - 2 == len2 && !my_strnicmp(str1, str2, len2) && !end_strcmp(str1, ".Z", 2))
+				entries = 1;
+			else if (len1 - 2 == len2 && !my_strnicmp(str1, str2, len2) && !end_strcmp(str1, ".z", 2))
 				entries = 1;
-			}
 		}
 
 		if (!*help_topic_list)
@@ -536,11 +499,14 @@ static	void	help_me (char *topics, char *args)
  * show the help file, and the default means to add the
  * stuff to the paused topic list..
  */
+if (!entries)
+	entries = g.gl_matchc;
+
 switch (entries)
 {
 	case -1:
 	{
-		help_put_it(no_help, "%s Error during help function: %s", thing_ansi, sys_errlist[errno]);
+		help_put_it(no_help, "*** Error during help function: %s", strerror(errno));
 		set_help_screen(NULL);
 		if (help_paused_first_call)
 		{
@@ -553,15 +519,15 @@ switch (entries)
 	}
 	case 0:
 	{
-		help_put_it(this_arg, "%s No help available on %s: Use ? for list of topics", thing_ansi, this_arg);
+		help_put_it(this_arg, "*** No help available on %s: Use ? for list of topics", this_arg);
 		if (!get_int_var(HELP_PROMPT_VAR))
 		{
 			set_help_screen(NULL);
 			break;
 		}
-		sprintf(tmp, "%s%sHelp? ", help_topic_list, *help_topic_list ? " " : empty_string);
-		if (!dumb)
-			add_wait_prompt(tmp, help_me, help_topic_list, WAIT_PROMPT_LINE);
+		sprintf(tmp, "%s%sHelp? ", help_topic_list, *help_topic_list ? space : empty_string);
+		if (!dumb_mode)
+			add_wait_prompt(tmp, help_me, help_topic_list, WAIT_PROMPT_LINE, 1);
 
 		if (help_paused_first_call)
 		{
@@ -570,47 +536,36 @@ switch (entries)
 			new_free(&help_paused_path);
 			new_free(&help_paused_name);
 		}
+	
 		break;
 	}
 	case 1:
 	{
-		sprintf(tmp, "%s/%s", path, namelist[0]->d_name);
-		stat_file(tmp, &stat_buf);
+		sprintf(tmp, "%s/%s", path, g.gl_pathv[0]);
+		stat(tmp, &stat_buf);
 		if (stat_buf.st_mode & S_IFDIR)
 		{
 			strcpy(path, tmp);
 			if (*help_topic_list)
-				strcat(help_topic_list, " ");
+				strcat(help_topic_list, space);
 
-			strcat(help_topic_list, namelist[0]->d_name);
-			if ((this_arg = next_arg(args, &args)) == (char *) 0)
+			strcat(help_topic_list, g.gl_pathv[0]);
+
+			if (!(this_arg = next_arg(args, &args)))
 			{
 				help_paused_first_call = 1;
 				malloc_strcpy(&help_paused_path, path);
-				malloc_strcpy(&help_paused_name, namelist[0]->d_name);
+				malloc_strcpy(&help_paused_name, g.gl_pathv[0]);
 				dont_pause_topic = -1;
 				this_arg = "?";
 			}
-			/*
-			 * Why do we free this here, like this?
-			 * Because we're about to continue, which means
-			 * we hop back up to the top of the loop:  We will
-			 * miss the cleanup that happens just after the end
- 			 * of this case statement, and so we must do the
- 			 * cleanup to avoid the memory leak.  We use free_cnt,
-			 * because it is the 'true' measure of what is in
-			 * the array (entries might have been modified above).
-			 */
-			for (i = 0; i < /*free_cnt*/ entries; i++)
-				free((char *)namelist[i]);
-			free((char *)namelist);
-			namelist = NULL;
+			bsd_globfree(&g);
 			continue;
 		}
 		else
 		{
-			help_topic(path, namelist[0]->d_name);
-			finished_help_paging = 0;	/* this is a big kludge */
+			help_topic(path, g.gl_pathv[0]);
+			finished_help_paging = 0;
 			break;
 		}
 	}
@@ -618,28 +573,20 @@ switch (entries)
 	{
 		help_show_directory = 1;
 		strcpy(paused_topic, help_topic_list);
-		help_pause_add_line("%s %s choices:", thing_ansi, help_topic_list);
+		help_pause_add_line("*** %s choices:", help_topic_list);
 		entry_size += 2;
-		cols = (CO - 10) / entry_size;
+		cols = (current_term->TI_cols - 10) / entry_size;
 
 		strcpy(buffer, empty_string);
 		cnt = 0;
 
 		for (i = 0; i < entries; i++)
 		{
-			/*
-			 * Here we remove the compression suffix (if any)
-			 * from the filename, since we're not interested in
-			 * it at this point.
-			 */
-			if (!end_strcmp(namelist[i]->d_name, ".gz", 3))
-				chop(namelist[i]->d_name, 3);
-
-			/*
-			 * Then we append the help topic to the current
-			 * output line.
-			 */
-			strcat(buffer, namelist[i]->d_name);
+			if (!end_strcmp(g.gl_pathv[i], ".gz", 3))
+				chop(g.gl_pathv[i], 3);
+			else if (!end_strcmp(g.gl_pathv[i], ".bz2", 4))
+				chop(g.gl_pathv[i], 4);
+			strcat(buffer, g.gl_pathv[i]);
 
 			/*
 			 * Since we already know how many columns each
@@ -661,7 +608,7 @@ switch (entries)
 			 * do adjust for compression extension.
 			 */
 			else
-				strextend(buffer, ' ', entry_size - strlen(namelist[i]->d_name));
+				strextend(buffer, ' ', entry_size - strlen(g.gl_pathv[i]));
 		}
 
 		help_pause_add_line("%s", buffer);
@@ -683,34 +630,24 @@ switch (entries)
 /* end of reformatting */
 
 
-		/*
-		 * We only get here if everything happened ok.  free_cnt
-		 * can be -1, 0, or a positive value here, and we know that
-		 * if we get here, we havent free()d it yet.  So we free it
-		 * here and exit the while loop.
-		 */
-		if (namelist)
-		{
-			for (i = 0; i < free_cnt; i++)
-				free((char *)namelist[i]);
-			free((char *)namelist);
-		}
+		bsd_globfree(&g);
 		break;
 	}
+
 	/*
 	 * This one is for when there was never a topic and the prompt
 	 * never got a topic..  and help_screen was never reset..
 	 * phone, jan 1993.
 	 */
 	if (!*help_topic_list && finished_help_paging)
-		set_help_screen(NULL);
+		set_help_screen((Screen *) 0);
 }
 
 /*
  * help: the HELP command, gives help listings for any and all topics out
  * there 
  */
-extern void help (char *command, char *args, char *subargs)
+BUILT_IN_COMMAND(epichelp)
 {
 	char	*help_path;
 
@@ -725,54 +662,55 @@ extern void help (char *command, char *args, char *subargs)
 	 * ever it maybe.  Once we have worked this out, if we are using
 	 * a help window, set it up properly.
 	 */
-
 	help_path = get_string_var(HELP_PATH_VAR);
 
-	if (!(help_path && *help_path && !access(help_path, R_OK | X_OK)))
+	if (!help_path || !*help_path || access(help_path, R_OK | X_OK))
 	{
-		help_put_it(no_help, "%s HELP_PATH variable not set or set to an invalid path", thing_ansi);
+		help_put_it(no_help, "*** HELP_PATH variable not set or set to an invalid path");
 		return;
 	}
 
-	/*
-	 * Here, if we are using the help files locally, we must ensure that
-	 * we aren't doing HELP in a more than one screen - phone, jan 1993.
-	 */
+	/* Allow us to wait until help is finished */
+	if (!my_strnicmp(args, "-wait", 2))
+	{
+		while (help_screen)
+			io("help");
+		return;
+	}
 
-	if (help_path && help_screen && help_screen != current_screen)
+	if (help_path && help_screen && help_screen != current_window->screen)
 	{
 		say("You may not run help in two screens");
 		return;
 	}
-	help_screen = current_screen;
-	help_window = NULL;
-	help_me(empty_string, (args && *args) ? args : "?");
 
+	help_screen = current_window->screen;
+	help_window = (Window *) 0;
+	help_me(empty_string, (args && *args) ? args : "?");
 }
 
 
 
 
-static	void create_help_window _((void))
+static	void create_help_window (void)
 {
 	if (help_window)
 		return;
 
-	if (!dumb && get_int_var(HELP_WINDOW_VAR))
+	if (!dumb_mode && get_int_var(HELP_WINDOW_VAR))
 	{
 		use_help_window = 1;
-		help_window = new_window();
-
+		help_window = new_window(current_window->screen);
 		help_window->hold_mode = OFF;
+		help_window->window_level = LOG_HELP;
 		update_all_windows();
 	}
 	else
-		help_window = curr_scr_win;
+		help_window = current_window;
 }
 
 
 
-
 static	void	set_help_screen (Screen *screen)
 {
 	help_screen = screen;
@@ -786,7 +724,40 @@ static	void	set_help_screen (Screen *screen)
 			delete_window(help_window);
 			window_display = display;
 		}
-		help_window = NULL;
+		help_window = (Window *) 0;
 		update_all_windows();
 	}
 }
+
+static	void	help_put_it	(const char *topic, const char *format, ...)
+{
+	char putbuf[BIG_BUFFER_SIZE * 3 + 1];
+
+	if (format)
+	{
+		va_list args;
+		va_start (args, format);
+		vsnprintf(putbuf, BIG_BUFFER_SIZE * 3, format, args);
+		va_end(args);
+
+		if (do_hook(HELP_LIST, "%s %s", topic, putbuf))
+		{
+			int old_level = who_level;
+			Window *old_target_window = target_window;
+
+			/*
+			 * LOG_HELP is a completely bogus mode.  We use
+			 * it only to make sure that the current level is
+			 * not LOG_CURRENT, so that the to_window will stick.
+			 */
+			who_level = LOG_HELP;
+			if (help_window)
+				target_window = help_window;
+			add_to_screen(putbuf);
+			target_window = old_target_window;
+			who_level = old_level;
+		}
+	}
+}
+#endif
+
